<?php

/**
 * @author Steffen Wirsching, Christoph Staudt
 * @copyright 2010
 * @project PSI-Online
 * @status Fast Fertig programmiert
 * @to(do) utf8_decode?
 * @test cDB_test.php
 * @done Referenz wurde entfernt, Select Problem mit falschen keys wie DATE_FORMAT(`datum`, '%d.%m.%Y') durch as $colname behoben
 */

/**
 * @about Stellt Funktionen zur Verwaltung der im Konstruktor angegebenen Tabelle zur Verfgung
 */
interface iDB {
    /**
     * @param object, string
     * @return none
     * @about Erstellt ein neues DB-Objekt
     */
    function __construct($o_db, $s_table);
    /**
     * @param [boolean]
     * @return array
     * @about Gibt die Daten der Tabelle zurck. Wird true bergeben, werden nur die Spaltennamen zurckgegeben
     */
    function getdbinfo($b_colnamesonly = false);
    /**
     * @param [id],[array]
     * @return array || none
     * @about Fhrt eine SELECT-Abfrage auf der Tabelle aus. Wird eine id angegeben, wird nur der Datensatz mit der entsprechenden id ausgew?hlt. Mit Hilfe des dbfuncs-Parameters k?nnen Sie auf jedes selektierte Feld MySQL-Funktionen wie z.B. AS anwenden. zurckgegeben wird ein mehr-dimensionales Array der Form [row:int]=>[field:string]=>[value:mixed]
     */
    function Select($json_options = '');
    /**
     * @param  sting, array, [array]
     * @return array
     * @deprecated Diese Funktion ist depracted und sollte nicht verwendet werden
     * @about Fhrt eine Select Abfrage mit der Bedingung $s_condition aus. $a_variables[name] = value Der name muss eine vorhandene Tabellenspalte sein, und der Wert muss der Inhalt der Spalte sein.
     */
    function SelByCon($s_condition, $a_variables, $a_dbfuncs = array());
    /**
     * Fgt das bergebene Array (s. @array) als Datenstz(e) in die Tabelle ein.
     * Mit Hilfe von dbfuncs knnen statt festen Werten auch MySQL-Funktionen wie z.B. NOW() als Wert verwendet werden.
     * @param array $a_values Das Array kann ein- oder zwei-dimensional bergeben werden. Eindimensionale Arrays m?ssen von der Form [field:int]=>[value:mixed], zweidimensionale von der Form [row:int]=>[field:int]=>[value:mixed] sein. Eindimensionale Arrays erzeugen nur einen Eintrag in der Tabelle, w?hrend zweidimensionale so viele Eintr?ge in der Tabelle erzeugen, wie die erste Dimension Elemente hat. Eine Vermischung der Varianten ist nicht erlaubt! Dabei werden die Tabellenfelder aufsteigend gezhlt, mit 0 beginnend, die ID aber ?berspringend.
     * @param array
     * @return boolean 
     */
    function Insert($a_values, $db_funcs = array());
    /**
     * 
     * berschreibt die Eintrge mit den bergebenen Werten in dem Datensatz, der angegebenen ID. Ist ID,
     * werden ALLE Datenstze mit den Eintr?gen ?berschrieben.
     * Mit Hilfe von dbfuncs k?nnen statt festen Werten auch MySQL-Funktionen wie z.B. NOW() als Wert verwendet werden.
     * 
     * @param array $a_toupdate
     * @param [string $json_options Ein JSON Objekt, dass die Bedingungen angibt]
     * @return booleans
     *
     */
    function Update($a_toupdate, $json_options = '');
    /**
     * @param [int], [string]
     * @return boolean
     * Lscht die Datenstze von der Tabelle, die spezifiziert werden durch entweder nur (XOR) die angegebene ID [where:null], nur (XOR) die angegebene (selbst zu erstellende) WHERE-Klausel [id:null] oder durch einen von beiden (OR). Sind sowohl ID als auch WHERE-Parameter null, werden ALLE Datenstze endgltig gelscht!
     */
    function Delete($json_options = '');
    /**
     * @param string, string [,string]
     * @return array
     * @about Liest die Spalten $s_field1 und $s_field2 aus der Datenbank aus Sortiert sie aufsteigend nach $s_field2. Optional kann eine where Bedingung ohne fremdwerte angegeben werden
     */
    function formdata($s_field1,$s_field2, $s_where);
    /**
     * @param none
     * @return int
     * @about Gibt die ID des letzten INSERT zurck
     */
    function getinsertid();
    /**
     * @param string, mysqli object
     * @return array
     * @about Fhrt eine Query aus und gibt das Ergebnis als array zurck
     */
    static function query($sql, $o_db);
    /**
     * @param string, mysqli object
     * @return array
     * @ Fhrt einen Query aus, der nur ein Resultat zurckgibt.
     */
    static function query_uq($sql, $o_db);
}
class cDB implements iDB {
	private $o_db;
	private $a_colnames;
	private $s_table;
    private $a_types;
    private $a_coles;
    function __construct($o_db, $s_table) {
        //Parameter berprfen
        if (!$o_db instanceof mysqli) {
            throw new Exception('$o_db muss eine Instanz des Mysqli Objekts sein.');
        }
        if (!is_string($s_table)) {
            throw new Exception('$s_table muss ein string sein.');
        }
        $this->o_db = $o_db;
        $this->s_table = $s_table;
        $this->a_colnames = $this->getdbinfo(true);
        $this->a_coles = $this->getdbinfo();
        foreach ($this->a_coles as $a_cole) {
            $this->a_types[$a_cole['Field']] = $this->getFieldType($a_cole['Type']);
        }
        if (is_array($this->a_colnames) !== true) {
            throw new Exception('Fehler in cDB::getdbinfo(true)! Mgliche Ursache: Tabelle existiert nicht.');
        }
    }

    private function getFieldType($s_type) {
        if (preg_match('/int/i', $s_type) !== 0) {
            return 'i';
        } else {
            return 's';
        }
    }

    function getdbinfo($b_colnamesonly = false, $b_id = false) {
        //Parameter berprfen
        if (!is_bool($b_colnamesonly)) {
            throw new Exception('$b_colnamesonly muss eine boolesche Variable sein.');
        }
        $sql = 'SHOW FULL COLUMNS FROM `' . $this->s_table . '`';
        $result = $this->o_db->query($sql);
        if (!$result) {
            throw new Exception('Die angegebene Tabelle exsistiert nicht!', 0);
        }
        $return = array();
        while ($row = $result->fetch_assoc()) {
            if ($b_colnamesonly) {
                $return[] = $row['Field'];
            } else {
                $return[] = $row;
            }
        }
        return $return;
    }

    function Select($i_id = null, $json_options = '') {
        
        list($a_dbfuncs, $query_option, $a_bindparam, $b_2d) = $this->get_options($i_id, $json_options, true);

        // Felder die selektiert werden sollen bestimmen
        $s_toselect = '';
        foreach ($this->a_colnames as $colname) {
            if (isset($a_dbfuncs[$colname])) {
                $colname == end($this->a_colnames) ? $s_toselect .= $a_dbfuncs[$colname] . ' as ' . $colname : $s_toselect .= $a_dbfuncs[$colname] . ' as ' . $colname . ',';
            } else {
                $colname == end($this->a_colnames) ? $s_toselect .= $colname : $s_toselect .= $colname . ',';
            }
        }

        $sql = 'SELECT ' . $s_toselect . ' FROM `' . $this->s_table . '` ' . $query_option;
        // Wenn Variablen gebunden werden mssen
        if (isset($a_bindparam) === true and is_array($a_bindparam) === true) {
            if (!$stmt = $this->o_db->prepare($sql)) {
                throw new Exception($this->o_db->error);
            }
            eval($a_bindparam['eval']);
            if (!$stmt->execute()) {
                die($stmt->error);
            }
            $stmt->store_result();
            if ($stmt->num_rows < 1) {
                return false;
            }
            eval('$stmt->bind_result(' . $this->getbindvariables() . ');');
            if ($stmt->num_rows < 1) {
                return false;
            }
            if ($stmt->num_rows == 1) {
                $return = array();
                $stmt->fetch();
                foreach ($this->a_colnames as $colname) {
                    $return[$colname] = ${$colname};
                }
                if ($b_2d) {
                    $return = array($return);
                }
            } else {
                $i = 0;
                $return = array();
                while ($stmt->fetch()) {
                    $return[$i] = array();
                    foreach ($this->a_colnames as $colname) {
                        $return[$i][$colname] = ${$colname};
                    }
                    $i++;
                }
                unset($i);
            }
            return $return;
            $stmt->close();
        } else {
            $result = $this->o_db->query($sql);
            if (!$result) {
                throw new Exception('Etwas stimmte mit dem Query (' . $sql . ') nicht: ' . $this->o_db->error);
            }
            $return = array();
            if ($result->num_rows < 1) {
                return false;
            }
            while (($row = $result->fetch_assoc())) {
                $return[] = $row;
            }
        }
        return $return;
    }

    function SelByCon($s_condition, $a_variables, $a_dbfuncs = array()) {
        //Parameter berprfen
        if (is_string($s_condition) !== true) {
            throw new Exception('Die Bedingung muss ein string sein.');
        }
        if (is_array($a_variables) !== true) {
            throw new Exception('Die Bedingung muss in einem Array angegeben werden.');
        }
        if (count($a_variables) < 1) {
            throw new Exception('Es muss mindestens eine Bedingung angegeben werden.');
        }
        if (is_array($a_dbfuncs) !== true) {
            throw new Exception('Die Mysql funktionen mssen in einem Array bergeben werden.');
        }
        $s_toselect = '';
        foreach ($this->a_colnames as $colname) {
            if (isset($a_dbfuncs[$colname])) {
                $colname == end($this->a_colnames) ? $s_toselect .= $a_dbfuncs[$colname] . ' as ' . $colname : $s_toselect .= $a_dbfuncs[$colname] . ' as ' . $colname . ',';
            } else {
                $colname == end($this->a_colnames) ? $s_toselect .= $colname : $s_toselect .= $colname . ',';
            }
        }
        $sql = 'SELECT ' . $s_toselect . ' FROM `' . $this->s_table . '`';
        $sql .= ' WHERE ' . $s_condition;
        if (!$stmt = $this->o_db->prepare($sql)) {
            throw new Exception($this->o_db->error);
        }
        $s_types = '';
        $s_variables = '';
        $a_coles = $this->getdbinfo();
        foreach ($a_variables as $name => $value) {
            $s_types .= $this->a_types[$name];
            $s_variables .= ',$' . $name;
            ${$name} = $value;
        }
        eval('$stmt->bind_param(\'' . $s_types . '\'' . $s_variables . ');');
        unset($s_types, $s_variables);
        if (!$stmt->execute()) {
            throw new Exception($stmt->error);
        }
        $stmt->store_result();
        eval('$stmt->bind_result(' . $this->getbindvariables() . ');');
        if ($stmt->num_rows < 1) {
            return false;
        }
        if ($stmt->num_rows == 1) {
            $return = array();
            $stmt->fetch();
            foreach ($this->a_colnames as $colname) {
                $return[$colname] = ${$colname};
            }
        } else {
            $i = 0;
            $return = array();
            while ($stmt->fetch()) {
                $return[$i] = array();
                foreach ($this->a_colnames as $colname) {
                    $return[$i][$colname] = ${$colname};
                }
                $i++;
            }
            unset($i);
        }
        $stmt->close();
        return $return;
    }

// Ende von SelbyCon()


    private function ins($a_values, $a_dbfuncs) {
        $a_coles = $this->getdbinfo();
        $index = 0;
        $s_values = '';
        $s_variables = '';
        $types = '';
        $s_cols = '';
        //Erzeugen der Variablen
        foreach ($a_coles as $a_col) {
            if (($a_col['Key']) == 'PRI') {
                continue;
            }
            $s_cols .= '`' . $a_col['Field'] . '`,';
            if (isset($a_dbfuncs[$a_col['Field']])) {
                $s_values .= $a_dbfuncs[$a_col['Field']] . ",";
            } else {
                $s_values .= '?,';
                $s_variables .= '$' . $a_col['Field'] . ",";
                $types .= $this->getFieldType($a_col['Type']);
                ${$a_col['Field']} = $a_values[$index];
                $index++;
            }
        }
        unset($a_coles, $a_col);

        //Query erstellen
        $sql = 'INSERT INTO ' . $this->s_table . '(' . rtrim($s_cols, ",") . ') VALUES(' . rtrim($s_values, ",") . ');';
        unset($s_cols, $s_values);

        //Query ausfhren
        if (!$stmt = $this->o_db->prepare($sql)) {
            throw new Exception($this->o_db->error);
        }
        eval('$stmt->bind_param(\'' . $types . '\', ' . rtrim($s_variables, ",") . ');');
        unset($types, $s_variables);
        if (!$stmt->execute()) {
            throw new Exception($stmt->error);
        }
        $stmt->close();
        return true;
    }

    function Insert($a_values, $a_dbfuncs = array()) {
        //Parameter berprfen
        if (!is_array($a_values)) {
            throw new Exception('$a_values muss ein array sein!');
        }
        $b_one = false; //(noch) nicht als eindimensional eingestuft
        $b_two = false; //(noch) nicht als zweidimensional eingestuft
        foreach ($a_values as $i_key => $m_value) {
            if (!is_int($i_key)) {
                throw new Exception('$a_values muss numerisch sein!');
            }
            if (is_array($m_value) && $b_one) { //Wenn Zweidimensionalitt erkannt wird, obwohl das Array schon als eindimensional eingestuft wurde
                throw new Exception('$a_values muss entweder ein- oder zwei-dimensional aufgebaut sein. Eine Vermischung der Varianten ist unzulssig!');
            } elseif (is_array($m_value)) { //Wenn Zweidimensionalitt erkannt wird und das Array entweder uneingestuft oder als zweidimensional eingestuft vorliegt
                $b_two = true; //als zweidimensional einstufen
                $a_value = $m_value;
                unset($m_value);
                foreach ($a_value as $i_key => $m_value) {
                    if (!is_int($i_key)) {
                        throw new Exception('Auch die zweite Dimension von $a_values muss numerisch sein!');
                    }
                }
            } elseif ($b_two) { //Wenn Eindimensionalitt erkannt wird, obwohl das Array schon als zweidimensional eingestuft wurde
                throw new Exception('$a_values muss entweder ein- oder zwei-dimensional aufgebaut sein. Eine Vermischung der Variaten ist unzulssig!');
            } else { //Wenn Eindimensionalitt erkannt wird und das Array entweder uneingestuft oder als eindimensional eingestuft vorliegt
                $b_one = true; //als eindimensional einstufen
            }
        }

        if (!is_array($a_dbfuncs)) {
            throw new Exception('$a_dbfuncs muss ein array sein!');
        }
        if ($b_one) {
            return $this->ins($a_values, $a_dbfuncs);
        } elseif ($b_two) {
            $b_dead = false;
            foreach ($a_values as $i_row => $a_newvalues) {
                if (!$this->ins($a_newvalues, $a_dbfuncs)) {
                    $b_dead = true;
                    throw new Exception('Einfgen des Datensatzes ' . $i_row . ' fehlgeschlagen! Weiter mit nchstem Datensatz');
                    continue;
                }
            }
            if (!$b_dead) {
                return true;
            }
        }
    }

    function Update($a_toupdate, $i_id = null, $json_options = '') {
        //Parameter berprfen
        if (!is_array($a_toupdate)) {
            throw new Exception('$a_values muss ein Array sein!');
        }

        list($a_dbfuncs, $query_option, $a_bindparam) = $this->get_options($i_id, $json_options);
        //Variablen erzeugen
        $s_cols = '';
        $s_types = '';
        $s_variables = '';

        // Felder bestimmen
        foreach ($a_toupdate as $s_name => $value) {
            $s_cols .= '`' . $s_name . '` = ?,';
            $s_types .= $this->a_types[$s_name];
            ${$s_name} = $value;
            $s_variables .= '$' . $s_name . ',';
        }
        unset($s_name, $value);

        //Feld Funktionen hinzufgen
        foreach ($a_dbfuncs as $s_field => $s_func) {
            $s_cols .= '`' . $s_field . '` = ' . $s_func . ',';
        }
        unset($s_field, $s_func);

        //Query erstellen
        $sql = 'UPDATE `' . $this->s_table . '` SET ' . rtrim($s_cols, ",") . ' ' . $query_option;
        unset($s_cols);

        //Query senden und verarbeiten
        if (!$stmt = $this->o_db->prepare($sql)) {
            die($this->o_db->error);
        }

        // Parameter binden
        if (isset($a_bindparam) === true and is_array($a_bindparam) === true) {
            $eval = '$stmt->bind_param(\'' . $s_types . $a_bindparam['types'] . '\', ' . rtrim($s_variables, ",") . $a_bindparam['variables'] . ');';
        } else {
            $eval = '$stmt->bind_param(\'' . $s_types . '\', ' . rtrim($s_variables, ",") . ');';
        }
        eval($eval);
        unset($types, $a_bindparam, $s_variables);

        if (!$stmt->execute()) {
            throw new Exception($stmt->error);
        }
        if ($stmt->affected_rows < 1) {
            return false;
        }
        $stmt->close();
        return true;
    }

    function Delete($i_id = null, $json_options = '') {
        list($a_dbfuncs, $query_option, $a_bindparam) = $this->get_options($i_id, $json_options);
        $sql = 'DELETE FROM `' . $this->s_table . '` ' . $query_option;

        if (!$stmt = $this->o_db->prepare($sql)) {
            throw new Exception($this->o_db->error);
        }

        // Mgliche Parameter binden
        if (isset($a_bindparam) === true and is_array($a_bindparam) === true) {
            eval($a_bindparam['eval']);
        }

        if (!$stmt->execute()) {
            throw new Exception($stmt->error);
        }
        $stmt->close();
        return true;
    }
    
    function getbindvariables($b_array = false){
  		$s_variables = '';
        if ($b_array){
            foreach ($this->a_colnames  as $s_colname){
    				$s_variables .= '$row["'.$s_colname.'"],';;
    		}
            return rtrim($s_variables, ',');
        }
        else
        {
    		foreach ($this->a_colnames  as $s_colname){
    			if ($s_colname == end($this->a_colnames)) {
    				$s_variables .= '$'.$s_colname.'';
    			}else {
    				$s_variables .= '$'.$s_colname.',';;
    			}
    		}
            return $s_variables;
        }
    }
    
    function formdata($s_field1, $s_field2, $s_where = ''){
        $sql = 'SELECT `'.$this->s_table.'`.`'.$s_field1.'`, `'.$this->s_table.'`.`'.$s_field2.'` FROM `'.$this->s_table.'` '.$s_where.' ORDER BY `'.$s_field2.'`';
        $result = $this->o_db->query($sql);
        if (!$result) {
            die ('Etwas stimmte mit dem Query nicht: '.$this->o_db->error);
        }
        if($result->num_rows < 1){
            return array();
        }
        $return = array();
        while ($row = $result->fetch_assoc()) {  // NULL ist quivalent zu false
            $return[$row[$s_field1]] = $row[$s_field2];
        }
        return $return;
    }
    
    function getinsertid(){
        return $this->o_db->insert_id;
    }

    private function get_options($i_id, $json_options, $b_select = false){
        if (!is_string($json_options)) {
            throw new Exception('$json_option muss ein string sein, Tabelle:' . $this->s_table);
        }
        if (is_numeric($i_id) === false) {
            if (is_string($i_id)) {
                $json_options = $i_id;
                $i_id = null;
            } else {
                if (!empty($i_id)) {
                    throw new Exception('Die id muss eine Zahl oder ein JSON string sein, Tabelle:'. $this->s_table .' Die ID:'.$i_id.' JSON:'.$json_options );
                }
                else{
                    return;
                }
            }
        }

        // Optionen bestimmen
        $a_options = json_decode($json_options, true);
        $b_2d = false;
        if (isset($a_options['b2d']) and ($a_options['b2d'] == true)) {
            $b_2d = true;
            unset($a_options['b2d']);
        }
        if (isset($i_id) and $i_id != null) {
            $a_options['id'] = $i_id;
        }
        if (is_array($a_options)) {
            list($a_dbfuncs, $query_option, $a_bindparam) = $this->options2array($a_options);
        } else {
            if ($json_options != '') {
                throw new Exception('Ihr JSON war nicht gltig:' . json_last_error());
            }
            $a_dbfuncs = array();
            $query_option = '';
        }
        if ($b_select == true) {
            return array($a_dbfuncs,$query_option,$a_bindparam,$b_2d);
        }
        return (array($a_dbfuncs,$query_option,$a_bindparam));

    }
    
    private function is_valid_field($s_name){
        if (array_search($s_name, $this->a_colnames) !== false){
            return true;
        }
        return false;
    }

    private function options2array($a_options){
        // Zum berprfen, ob WHERE noch gesetzt werden muss und ob parameter gebunden werden mssen
        $b_where = false;
        $b_eval = false;
        
        // einzelne sql teile
        $s_where = '';
        $s_sort = '';
        $s_limit = '';
        $s_group = '';      
        
        // Bind Param Vairablen
        $s_types = '';
        $s_variables = '';
        $a_variables = array();
        
        $a_dbfuncs = array();
        foreach ($a_options as $s_optionname => $a_option){
            switch ($s_optionname){
                case $this->is_valid_field($s_optionname):
                    $b_eval = true;
                    $s_types .= $this->a_types[$s_optionname];
                    ${$s_optionname} = $a_option;
                    $a_variables[$s_optionname] = &${$s_optionname};
                    $s_variables .= ',$a_bindparam[\'values\'][\''.$s_optionname.'\']';

                    if($b_where){
                     $s_where  .= ' AND `'.$s_optionname.'` = ?';
                    }
                    else
                    {
                        $s_where .= 'WHERE `'.$s_optionname.'` = ?';
                        $b_where = true;
                    }
                    break;
                case 'where':
                    if($b_where){
                        $s_where = $a_option;
                    }
                    else
                    {
                        $s_where .= 'WHERE '.$a_option;
                        $b_where = true;
                    }
                    break;
                case 'sort':
                    if (is_array($a_option) === true){
                        $s_sort = ' ORDER BY `'.$a_option[0].'` '.$a_option[1];
                    }
                    else
                    {
                        $s_sort = ' ORDER BY `'.$a_option.'`';
                    }
                    break;
                case 'group':
                    $s_group = ' GROUP BY `'.$a_option.'`';
                    break;
                case 'func':
                    $a_dbfuncs[$a_option[0]] = $a_option[1];
                    break;
                case 'now':
                    $a_dbfuncs[$a_option] = 'NOW()';
                    break;
                case 'date_format':
                    $a_dbfuncs[$a_option[0]] = ' DATE_FORMAT(`'.$a_option[0].'`, \''.$a_option[1].'\')';
                    break;
                case 'limit':
                    if (is_array($a_option) === true){
                       $s_limit = ' LIMIT '.$a_option[0].','.$a_option[1];
                    }
                    else
                    {
                        $s_limit = ' LIMIT '.$a_option[0];
                    }
                    break;
                default: 
                   throw new Exception('Ihre Angaben zu den optionen des Querys waren falsch:'.$s_optionname);
                    
            }
        }
        $sql = $s_where.$s_group.$s_sort.$s_limit;
        if ($b_eval){
            $a_bindparam = array('eval' => '$stmt->bind_param(\''.$s_types.'\''.$s_variables.');', 'values' => $a_variables, 'types' => $s_types, 'variables' => $s_variables);
            return array($a_dbfuncs, $sql, $a_bindparam);
        }
        return array($a_dbfuncs,$sql, null);
    }

    static function query($sql, $o_db){

        if (!$o_db instanceof mysqli) {
            throw new Exception('$o_db muss ein mysqli Objekt sein');
        }
        $result = $o_db->query($sql);
        if (!$result) {
            die ('Etwas stimmte mit dem Query nicht: '.$o_db->error);
        }
        $return = array();
        while ($row = $result->fetch_assoc()) {
            $return[] = $row;
        }
        $result->close();
        return $return;
    }

    static function  query_uq($sql, $o_db) {
        if (!$o_db instanceof mysqli) {
            throw new Exception('$o_db muss ein mysqli Objekt sein');
        }
        $result = $o_db->query($sql);
        if (!$result) {
            die ('Etwas stimmte mit dem Query nicht: '.$o_db->error);
        }
        if ($result->num_rows < 1) {
            return false;
        }
        if ($result->num_rows > 1){
            return false;
        }
        return $result->fetch_assoc();
    }
}
?>