<?php

/**
 * Description of cDBForm
 *
 * @author Christoph
 */
class cDBForm {

    private $tablename;
    private $form;
    private $db_object;
    private $o_db;
    private $label2field;
    private $db_funcs;
    private $b_edit;
    private $i_id;

    /**
     * Erzeugt eine neue Instanz des cDBForm-objekts
     *
     * @param mysqli $o_db mysqli Objekt
     * @param string $tablename Name der Tabelle
     *
     * @return none
     */
    function __construct($o_db, $tablename) {
        $this->o_db = $o_db;
        $this->tablename = $tablename;
        $this->db_object = new cDB($o_db, $tablename);
        $this->label2field = array();
        $this->db_funcs = array();
    }

    /**
     * Erzeugt ein Formularobjekt fr die angegebene Tabelle
     *
     * @param array $a_fieldoptions
     * @param bool $i_id Die id des Eintrages der bearbeitet werden soll
     *
     * @return none
     */
    function create_form($s_action, $s_legend, $s_name, $s_class, $a_fieldoptions = array(), $i_id = null) {
        if (is_int($i_id)) {
            $this->b_edit = true;
            $this->i_id = $i_id;
            $a_data = $this->db_object->Select($i_id);
            if ($a_data === false) {
                throw new Exception('Die angegebene ID was leider ungltig.');
            }
        }
        
        $a_columns = $this->db_object->getdbinfo();
        $this->form = new cForm();
        $this->form->start($s_action, $s_legend, $s_name, $s_class);
        foreach ($a_columns as $a_column) {
            if ($a_column['Extra'] == 'auto_increment') {
                continue;
            }
            if (isset($a_fieldoptions[$a_column['Field']]['fieldtype'])) {
                $s_method = $a_fieldoptions[$a_column['Field']]['fieldtype'];
            } else {
                if (stripos($a_column['Type'], 'text')) {
                    $s_method = 'textarea';
                } else {
                    $s_method = 'text';
                }
            }
            $a_params = array();
            //Label setzen
            if (isset($a_fieldoptions[$a_column['Field']]['label'])) {
                $a_params[] = $a_fieldoptions[$a_column['Field']]['label'];
            } else {
                $a_params[] = ucfirst($a_column['Field']);
            }
            $this->label2field[$this->form->rightname($a_params[0])] = $a_column['Field'];

            //Beschreibung setzen
            if (isset($a_fieldoptions[$a_column['Field']]['beschreibung'])) {
                $a_params[] = $a_fieldoptions[$a_column['Field']]['beschreibung'];
            } else {
                $a_params[] = $a_column['Comment'];
            }

            $a_params[] = 'string';

            //Beschreibung Pflicht setzen
            if (isset($a_fieldoptions[$a_column['Field']]['pflicht'])) {
                $a_params[] = $a_fieldoptions[$a_column['Field']]['pflicht'];
            } else {
                if ($a_column['Null'] == 'NO') {
                    $a_params[] = true;
                } else {
                    $a_params[] = false;
                }
            }
            if ($this->b_edit) {
                $a_params[] = $a_data[$a_column['Field']];
            }
            call_user_func_array(array($this->form, $s_method), $a_params);
        }
        $this->form->submit('Speichern');
        $this->form->end();
    }

    /**
     * Prft, ob das Formular richtig abgeschickt wurde, falls Fehler aufgtreten sind wird dies gespeichert
     *
     * @return bool
     */
    function check_form() {
        if ($this->form->is_correctsend()) {
            return true;
        }
        return false;
    }

    /**
     * bertrgt die geprften Daten in die Datenbank
     *
     * @return none
     */
    function transfer_input() {
        if ($this->b_edit) {
            $a_values = array();
            foreach ($this->label2field as $label => $field) {
                $a_values[$field] = $this->form->a_input[$this->form->rightname($label)];
            }
            $this->db_object->Update($a_values, $this->i_id);
        } else {
            $a_values = array();
            foreach ($this->label2field as $label => $field) {
                $a_values[] = $this->form->a_input[$this->form->rightname($label)];
            }
            $this->db_object->Insert($a_values, $this->db_funcs);
        }
    }

    /**
     * Gibt das $data arrayzurck das fr das Templatesystem bentigt wird.
     */
    function get_data() {
        if ($this->form->b_error) {
            $data = array('template' => 'form_error.tpl.php', 'form' => $this->form, 'errors' => $this->form->a_errors);
            return $data;
        }
        $data = array('template' => 'form.tpl.php', 'form' => $this->form);
        return $data;
    }

}

?>
