<?php

/**
 * @author Christoph Staudt
 * @ueberarbeitet Steffen Wirsching
 * @copyright 2010-2011
 */
interface iForm {

    //Interna
    /**
     * @return none
     * @about Konstruktor
     */
    public function __construct();

    /**
     * @return none
     * @about Startet das Formular
     */
    public function start($s_action, $s_legend, $s_name, $s_class = '', $s_complete =
    '');

    /**
     * @return none
     * @about Beendet das Formular (zur Ausgabe muss p_form aufgerufen werden!)
     */
    public function end();

    /**
     * @return none
     * @about Gibt das zusammengestellte Formular aus oder gibt es zurck, wenn gewnscht
     */
    public function p_form($b_return = false);

    /**
     * @return bool (Exceptions thrown)
     * @about Prft, ob das Formular richtig ausgefllt wurde; geht nur mit POST
     */
    public function is_correctsend();

    //Formularfelder
    /**
     * @param s_case: values may be 'start', 'mid' (default) or 'end'
     * @return none
     * @about Fgt eine Gruppierung fr Formularfelder in das Formular ein
     */
    public function fieldset($s_legend, $s_case = 'mid');

    /**
     * @param type: values may be 'int', 'string', 'float', 'email', 'ip', 'url'
     * @return none
     * @about Fgt ein verstecktes Formularfeld in das Formular ein
     */
    public function hidden($s_name, $s_type, $s_value);

    /**
     * @param beschreibung: wird angezeigt, um dem User zu erklren, wofr das Feld steht
     * @param type: values may be 'int', 'string', 'float', 'email', 'ip', 'url'
     * @return none
     * @about Fgt ein normales Textfeld in das Formular ein
     */
    public function text($s_label, $s_beschreibung, $s_type, $b_pflicht = true, $s_value =
    '');

    /**
     * @param beschreibung: wird angezeigt, um dem User zu erklren, wofr das Feld steht
     * @param type: values may be 'int', 'string', 'float', 'email', 'ip', 'url'
     * @return none
     * @about Fgt ein Password-Textfeld in das Formular ein
     */
    public function password($s_label, $s_beschreibung, $s_type, $b_pflicht = true);

    /**
     * @param beschreibung: wird angezeigt, um dem User zu erklren, wofr das Feld steht
     * @param type: values may be 'int', 'string', 'float', 'email', 'ip', 'url'
     * @return none
     * @about Fgt eine Textarea in das Formular ein
     */
    public function textarea($s_label, $s_beschreibung, $s_type, $b_pflicht = true, $s_value = '');

    /**
     * @param beschreibung: wird angezeigt, um dem User zu erklren, wofr das Feld steht
     * @param data: assoziatives Array, das die Datenpaare spezifiziert
     * @param m_selected: gibt den Default-Wert fr das DropDown an
     * @return none
     * @about Fgt ein DropDown-Feld in das Formular ein
     */
    public function dropdown($s_label, $s_beschreibung, $a_data, $b_pflicht = true, $m_selected = 0);

    /**
     * @param beschreibung: wird angezeigt, um dem User zu erklren, wofr das Feld steht
     * @param data: assoziatives Array, das die Datenpaare spezifiziert
     * @param m_checked: gibt den von Anfang an ausgewhlten Radio-Button an
     * @return none
     * @about Fgt Radiobuttons in das Formular ein
     */
    public function radio($s_label, $s_beschreibung, $a_data, $b_pflicht = true, $m_checked =
    0);

    /**
     * @param beschreibung: wird angezeigt, um dem User zu erklren, wofr das Feld steht
     * @param data: assoziatives Array, das die Datenpaare spezifiziert
     * @param a_checked: gibt die von Anfang an ausgewhlten Checkboxen an
     * @return none
     * @about Fgt Checkboxen in das Formular ein
     */
    public function checkbox($s_label, $s_beschreibung, $a_data, $b_pflicht = true, $a_checked = array());

    /**
     * @return none
     * @about Gibt einen Submit-Button aus
     */
    public function submit($s_value);

    //CKEditor
    /**
     * @return none
     * @about Erstellt ein neues Formular mit einer Textarea und verwandelt diese in den WYSIWYG-Editor ckeditor
     */
    function ckeditor($s_name, $s_value = '', $js_toolbar = '');

    /**
     * @return none
     * @about Da der ckeditor ein eigenes Formular ist, kann sein Wert mit dieser Funktion ausgelesen werden
     */
    static function get_ckeditor($s_name);

    /**
     * Setzt ein Formularfeld
     *
     * @param string, bool, string, string
     * @return none
     */
    function set_field($s_id, $b_pflicht, $s_type, $s_name);
}

class cForm implements iForm {

    public $s_output;
    private $a_fields;
    private $s_submit;
    private $a_output;
    private $s_name;
    private $i_tagid;
    public $a_input = array();
    public $a_errors = array();
    public $b_error = false;
    public $b_upload;
    static $f_editcontent;
    static $f_editconcept;

    public function __construct($b_upload = false) {
        $this->a_formtags = array();
        $this->a_fields = array();
        $this->s_output = '';
        $this->i_tagid = 0;
        $this->b_upload = $b_upload;
    }

    public function start($s_action, $s_legend, $s_name, $s_class = '', $b_ajax = false, $s_complete = '', $s_method = 'POST') {
        ob_start();
        $this->s_name = $s_name;
        $this->s_class = $s_class;
        $s_id = $this->make_id('form');
        echo '<form action="' . $s_action . '" method="' . $s_method . '" id="' . $s_id . '" class="' . $this->s_class . '"';
        if ($this->b_upload)
        {
            echo 'enctype="multipart/form-data"';
        }
        if ($s_complete != '')
        {
            echo 'onsubmit="$(\'#form\').load(this.action + \' .toload\' , $(this).serializeArray(), ' . $s_complete . '); return false;"';
        }
        elseif ($b_ajax == true)
        {
            echo 'onsubmit="$(\'#form\').load(this.action + \' .toload\' , $(this).serializeArray()); return false;"';
        }
        echo '>' . "\r\n";
        $this->fieldset($s_legend, 'start');
        return $s_id;
    }

    public function end($b_printend = true) {
        if ($b_printend)
        {
            $this->fieldset('', 'end');
            echo '</form>';
        }
        $this->s_output = ob_get_clean();
    }

    public function p_form($b_return = false) {
        if ($b_return)
        {
            return $this->s_output;
        }
        else
        {
            echo $this->s_output;
            return true;
        }
    }

    public function is_correctsend() {
        // berprfen ob Daten per Post gesendet wurden
        if ('POST' != $_SERVER['REQUEST_METHOD'])
        {
            return false;
        }
        // Prfen ob der submitbutton gesetzt ist
        if (isset($_POST[$this->submit]) === false)
        {
            return false;
        }
        if (count($_POST) > count($this->a_fields))
        {
            throw new Exception('Bitte benutzen Sie das Formular', 1);
        }
        $this->b_error = false;
        $a_toupload = array();
        foreach ($this->a_fields as $s_id => $options) {
            if ((isset($_POST[$s_id]) === true) or ($options['type'] == 'upload'))
            {
//              if ((is_array($_POST[$s_id]) == true) and ($options['type'] != 'array of string') and
//                    ($options['type'] != 'array of int')) {
//                    foreach ($_POST[$s_id] as $box) {
//                        if (filter_var($box, FILTER_VALIDATE_INT) === false) {
//                            throw new Exception('Das Feld "' . $options['name'] .
//                                '" muss eine Zahl enthalten.', 1);
//                        }
//                    }
//                    $this->a_input[$this->rightname($options['name'])] = $_POST[$s_id];
//                } else {
                if (($options['type'] != 'upload') and !is_array($_POST[$s_id]) and (trim($_POST[$s_id]) == '') and ($options['pflicht']
                        === true))
                {
                    $this->b_error = true;
                    $this->a_errors[] = 'Das Feld "' . $options['name'] . '" ist ein Pflichtfeld und muss ausgefllt werden.';
                }
                switch ($options['type']) {
                    case 'int':
                        if (filter_var($_POST[$s_id], FILTER_VALIDATE_INT) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Das Feld "' . $options['name'] .
                                    '" muss eine Zahl enthalten.';
                        }
                        break;
                    case 'string':
                        $_POST[$s_id] = filter_var($_POST[$s_id], FILTER_SANITIZE_STRING);
                        break;
                    case 'ckcontent':
                        break;
                    case 'float':
                        if (filter_var($_POST[$s_id], FILTER_VALIDATE_FLOAT) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Das Feld "' . $options['name'] .
                                    '" muss eine Zahl vom Typ float enthalten';
                        }
                        break;
                    case 'email':
                        if (filter_var($_POST[$s_id], FILTER_VALIDATE_EMAIL) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Bitte geben sie im E-Mail Feld eine richtige E-Mail ein.';
                        }
                        break;
                    case 'ip':
                        if (filter_var($_POST[$s_id], FILTER_VALIDATE_IP) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Die angegebene IP ist falsch.';
                        }
                        break;
                    case 'url':
                        if (filter_var($_POST[$s_id], FILTER_VALIDATE_URL) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Die angegebene URL enthlt Fehler.';
                        }
                        break;
                    case 'array of int':
                        if (is_array($_POST[$s_id]) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Das Feld "' . $options['name'] . '" muss ein array sein.';
                            break;
                        }
                        foreach ($_POST[$s_id] as $value) {
                            if (filter_var($value, FILTER_VALIDATE_INT) === false)
                            {
                                $this->b_error = true;
                                $this->a_errors[] = 'Das Feld "' . $options['name'] . '" muss eine Zahl enthalten.';
                                break;
                            }
                        }
                        break;
                    case 'array of string':
                        if (is_array($_POST[$s_id]) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Das Feld "' . $options['name'] . '" muss ein array sein.';
                            break;
                        }
                        foreach ($_POST[$s_id] as $key => $value) {
                            $_POST[$s_id][$key] = filter_var($value, FILTER_SANITIZE_STRING);
                        }
                        break;
                    case 'upload':
                        if (isset($_FILES[$s_id]['name'], $_FILES[$s_id]['size']) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Die Datei konnte leider nicht hochgeladen werden';
                            break;
                        }
                        if ($_FILES[$s_id]['name'] == '')
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Die Datei ist ungltig.';
                            break;
                        }
                        if (is_numeric($_FILES[$s_id]['size']) === false)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Die Datei konnte leider nicht hochgeladen werden.';
                            break;
                        }
                        if ($_FILES[$s_id]['size'] > MAX_FILE_SIZE)
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'Die Datei ist zu gro.';
                            break;
                        }
                        $a_pathinfo = pathinfo($_FILES[$s_id]['name']);

                        $_POST[$s_id] = array();

                        if ($a_pathinfo['extension'] == '')
                        {
                            $this->b_error = true;
                            $this->a_errors[] = 'ngltige Dateierweiterung.';
                            break;
                        }

                        if ($options['b_upload'])
                        {
                            //Generiere Dateiname
                            $filename = md5(uniqid(rand(), TRUE));
                            while (file_exists($options['directory'] . $filename)) {
                                $filename = md5(uniqid(rand(), TRUE));
                            }

                            $uploadfile = $options['directory'] . $filename;
                            $a_toupload[] = array();
                            $a_toupload[key($a_toupload)]['tmp_name'] = $_FILES[$s_id]['tmp_name'];
                            $a_toupload[key($a_toupload)]['filename'] = $uploadfile;

                            $_POST[$s_id]['extension'] = $a_pathinfo['extension'];
                            $_POST[$s_id]['filename'] = $filename;
                        }
                        else
                        {
                            $_POST[$s_id]['tmp_name'] = $_FILES[$s_id]['tmp_name'];
                            $_POST[$s_id]['extension'] = $a_pathinfo['extension'];
                        }
                        break;
                    default:
                        throw new Exception('Kein gltiger Typ angegeben fr das Feld: ' . $options['name'],
                                1);
                        ;
                } //switch
                $this->a_input[$this->rightname($options['name'])] = $_POST[$s_id];

                //}
            }
            else
            {
                if ($options['pflicht'] === true)
                {
                    $this->b_error = true;
                    $this->a_errors[] = 'Das Feld "' . $s_id . '" ist ein Pflichtfeld und muss ausgefllt werden.';
                }
            }
        } // foreach

        if ($this->b_error == true)
        {
            return false;
        }

        foreach ($a_toupload as $a_upload) {
            if (!move_uploaded_file($a_upload['tmp_name'], $a_upload['filename']))
            {
                $this->b_error = true;
                $this->a_errors[] = 'Die Datei konnte nicht gespeichert werden.';
                return false;
            }
        }
        return true;
    }

//is_correctsend

    public function hidden($s_name, $s_type, $s_value) {
        $s_id = $this->make_id('hidden');
        $this->set_field($s_name, true, $s_type, $s_name);
        echo '<input type="hidden" name="' . $s_name . '" id="' . $s_id . '" value="' .
        $s_value . '" class="' . $this->s_class . '"/>' . "\r\n";
    }

    public function text($s_label, $s_beschreibung, $s_type, $b_pflicht = true, $s_value = '') {
        $s_id = $this->make_id('text');
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
        ':</label>';

        echo '<input type="text" name="' . $s_id . '" id="' . $s_id . '" value="' .
        htmlentities($s_value) . '" class="' . $this->s_class . '" ' . $this->
                get_onfocus($s_id, $s_beschreibung) . '/>' . "\r\n";
        echo '<br>';
    }

    public function date($s_label, $s_beschreibung, $s_type, $b_pflicht = true, $s_value = '') {
        $s_id = $this->make_id('date');
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
        ':</label>';

        echo '<input type="text" name="' . $s_id . '" id="' . $s_id . '" value="' .
        htmlentities($s_value) . '" class="' . $this->s_class . '" ' . $this->
                get_onfocus($s_id, $s_beschreibung) . '/>' . "\r\n";
        echo '<br>';
        cPage::s_addJavaScript('
	$(function() {
		$( "#' . $s_id . '" ).datepicker();
	});
	');
    }

    public function password($s_label, $s_beschreibung, $s_type, $b_pflicht = true) {
        $s_id = $this->make_id('passw');
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);
        echo '<label for="' . $s_id . '">' . htmlentities($s_label) .
        ':</label><input type="password" name="' . $s_id . '" id="' . $s_id . '" ' . $this->
                get_onfocus($s_id, $s_beschreibung) . ' class="' . $this->s_class . '"/>' . "\r\n";
        echo '<br>';
    }

    public function submit($s_value) {
        $s_id = $this->make_id('submit');
        $this->set_field($s_id, true, 'string', 'submit');
        $this->submit = $s_id;
        echo '<input type="hidden" name="' . $s_id . '" id="' . $s_id . '" value="' . $s_value .
        '" class="' . $this->s_class . '"/>' . "\r\n";
        echo '<input type="submit"  value="' . $s_value . '" class="' . $this->s_name .
        ' form-button" id="' . $s_id . '"/>' . "\r\n";
        echo '<div class="clearer"></div>';
        echo '<br>';
    }

    public function file($s_label, $b_upload = true, $s_directory = '') {

        $s_id = $this->make_id('file');
        $this->set_field($s_id, false, 'upload', $s_label);
        $this->a_fields[$s_id]['directory'] = $s_directory;
        $this->a_fields[$s_id]['b_upload'] = $b_upload;

        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
        ':</label>';

        echo '<input type="file" name="' . $s_id . '" id="' . $s_id . '" class="' . $this->
        s_class . '"/>' . "\r\n";
        echo '<br>';
        return $s_id;
    }

    public function fieldset($s_legend, $s_case = 'mid') {
        switch ($s_case) {
            case 'start':
                echo '<fieldset class="' . $this->s_class . '">' . "\r\n";
                echo '<legend class="' . $this->s_class . '">' . $s_legend . '</legend>' . "\r\n";
                break;
            case 'end':
                echo '</fieldset>' . "\r\n";
                break;
            case 'mid':
            default:
                echo '</fieldset>' . "\r\n";
                echo '<fieldset class="' . $this->s_class . '">' . "\r\n";
                echo '<legend class="' . $this->s_class . '">' . $s_legend . '</legend>' . "\r\n";
                break;
        }
    }

    public function textarea($s_label, $s_beschreibung, $s_type, $b_pflicht = true, $s_value = '') {
        $s_id = $this->make_id('textarea');
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
        ':</label><textarea name="' . $s_id . '" id="' . $s_id . '" rows="" cols="" ' .
        $this->get_onfocus($s_id, $s_beschreibung) . ' class="' . $this->s_class . '">' .
        htmlentities($s_value) . '</textarea>';
        echo '<br>';
    }

    public function dropdown($s_label, $s_beschreibung, $a_data, $b_pflicht = true, $m_selected = 0, $s_type = 'int') {
        $s_id = $this->make_id('dropdown');
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
        ':</label>' . "\r\n";
        echo '<select name="' . $s_id . '" id="' . $s_id . '" class="' . $this->s_class .
        '" ' . $this->get_onfocus($s_id, $s_beschreibung) . ' >';
        foreach ($a_data as $s_value => $s_text) {
            if ($m_selected == $s_value)
            {
                echo '<option value="' . $s_value . '" selected="selected">' . htmlentities($s_text) .
                "</option>\r\n";
            }
            else
            {
                echo '<option value="' . $s_value . '">' . htmlentities($s_text) . "</option>\r\n";
            }
        }
        unset($s_value, $s_text);
        echo "</select>\r\n";
        echo '<br>';
    }

    public function radio($s_label, $s_beschreibung, $a_data, $b_pflicht = true, $m_checked =
    0, $s_type = 'int') {
        $s_id = $this->make_id('radio');
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
        ':</label><br/>' . "\r\n";
        reset($a_data);
        list($s_value, $s_text) = each($a_data);
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_text) .
        ':</label>' . "\r\n";
        if ($m_checked == $s_value)
        {
            echo '<input type="radio" class="' . $this->s_class . '" name="' . $s_id .
            '" id="' . $s_id . '"  value="' . $s_value . '" checked="checked" ' . $this->
                    get_onfocus($s_id, $s_beschreibung) . '/>';
        }
        else
        {
            echo '<input type="radio" class="' . $this->s_class . '" name="' . $s_id .
            '" id="' . $s_id . '"  value="' . $s_value . '" ' . $this->get_onfocus($s_id, $s_beschreibung) .
            '/>';
        }
        while (list($s_value, $s_text) = each($a_data)) {
            echo '<label for="' . $s_id . $s_text . '" class="' . $this->s_class . '">' .
            htmlentities($s_text) . ':</label>' . "\r\n";
            if ($m_checked == $s_value)
            {
                echo '<input type="radio" style="" name="' . $s_id . '" id="' . $s_id . $s_text .
                '"  value="' . $s_value . '" checked="checked" class="' . $this->s_class . '" ' .
                $this->get_onfocus($s_id, $s_beschreibung) . '/>' . "\r\n";
            }
            else
            {
                echo '<input type="radio" style="" name="' . $s_id . '" id="' . $s_id . $s_text .
                '"  value="' . $s_value . '" class="' . $this->s_class . '" ' . $this->
                        get_onfocus($s_id, $s_beschreibung) . '/>' . "\r\n";
            }
        }
        unset($s_value, $s_text);
        echo '<br>';
    }

    public function checkbox($s_label, $s_beschreibung, $a_data, $b_pflicht = true, $a_checked = array(), $s_type = 'array of int') {
        //id holen
        $s_id = $this->make_id('check');

        // Daten zum berprfen setzen
        $this->set_field($s_id, $b_pflicht, $s_type, $s_label);

        //label ausgeben
        if (count($a_data) > 1)
        {
            echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_label) .
            ':</label><br/>' . "\r\n";
        }

        //Beginn der Schleife
        reset($a_data);
        list($s_value, $s_text) = each($a_data);
        //label fr option
        echo '<label for="' . $s_id . '" class="' . $this->s_class . '">' . htmlentities($s_text) .
        ':</label>' . "\r\n";
        if (array_search($s_value, $a_checked) !== false)
        {
            echo '<input type="checkbox" name="' . $s_id . '[]" id="' . $s_id . '"  value="' .
            $s_value . '" class="' . $this->s_class . '" ' . $this->get_onfocus($s_id, $s_beschreibung) .
            ' checked="checked"/>' . "\r\n";
        }
        else
        {
            echo '<input type="checkbox" name="' . $s_id . '[]" id="' . $s_id . '"  value="' .
            $s_value . '" class="' . $this->s_class . '" ' . $this->get_onfocus($s_id, $s_beschreibung) .
            '/>' . "\r\n";
        }
        while (list($s_value, $s_text) = each($a_data)) {
            echo '<label for="' . $s_id . $s_text . '" class="' . $this->s_class . '">' .
            htmlentities($s_text) . ':</label>' . "\r\n\r\n";
            if (array_search($s_value, $a_checked) !== false)
            {
                echo '<input type="checkbox" name="' . $s_id . '[]" id="' . $s_id . $s_text .
                '"  value="' . $s_value . '" class="' . $this->s_class . '" ' . $this->
                        get_onfocus($s_id, $s_beschreibung) . ' checked="checked"/>' . "\r\n";
            }
            else
            {
                echo '<input type="checkbox" name="' . $s_id . '[]" id="' . $s_id . $s_text .
                '"  value="' . $s_value . '" class="' . $this->s_class . '" ' . $this->
                        get_onfocus($s_id, $s_beschreibung) . '/>' . "\r\n";
            }
        }
        echo '<br>';
    }

    function uploadfile($i_sid, $s_name, $js_complete, $s_class = '', $s_id = '') {
        ob_start();
        $this->s_name = $s_name;
        $s_formid = $this->make_id('form');
        $s_hideid = $this->make_id('hide');
        $s_nameid = $this->make_id('name');
        $s_fileid = $this->make_id('file');
        $s_buttonid = $this->make_id('button');
        $s_loaderid = $this->make_id('loader')
?>
        <div id="<?php echo $s_id ?>">
            <form class="<?php echo $s_class ?>" action="upload_file.php" method="post" enctype="multipart/form-data" id="<?php echo $s_formid; ?>">
                <fieldset>
                    <legend>Datei hochladen</legend>
                    <label class="<?php echo $s_class ?>" for="<?php echo $s_nameid; ?>">Dateiname:</label>
                    <input class="<?php echo $s_class ?>" type="text" name="name" id="<?php echo $s_nameid; ?>"/>
                    <input class="<?php echo $s_class ?>" type="hidden" name="sid" value="<?php echo $i_sid; ?>" />
                    <span id="<?php echo $s_hideid; ?>">
                        <label class="<?php echo $s_class ?>" for="<?php echo $s_fileid; ?>">Datei:</label>
                        <input class="<?php echo $s_class ?>" type="file" name="userfile" id="<?php echo $s_fileid; ?>"/><br />
                        <input class="<?php echo $s_class ?> form-button" type="submit" name="<?php echo $s_buttonid; ?>" value="Hochladen" id="<?php echo $s_buttonid; ?>"/>
                    </span>
                </fieldset>
            </form>
            <div id="<?php echo $s_loaderid; ?>" style="display:none;">L&auml;dt...<br/><img src="ajax-loader.gif" border="0"/></div>
        </div>

<?php
        $s_js = "
                        $(function() {
                            // Formular abschicken
                            $('#$s_formid').submit(function(data) {
                                // Ajax Loader anzeigen
                                $('#$s_loaderid').show();
                                // Formular Elemente verstecken
                                $('#$s_hideid').hide();
                                // abschickendes Formular angeben
                                var submittingForm = $('#$s_formid');
                                // eindeutigen iFrame Namen generieren
                                var frameName = (\"upload\" + (new Date()).getTime());
                                // iFrame setzen
                                var uploadFrame = $('<iframe name=\"' + frameName + '\"></iframe>');
                                // iFrame verstecken
                                uploadFrame.css(\"display\", \"none\");
                                // iFrame Inhalt laden
                                uploadFrame.load(function(data) {
                                    // Timeout festlegen
                                    setTimeout(function() {
                                        // Rueckgabewert abfragen
                                        var response = uploadFrame.contents().find('#result').html();
                                        $js_complete
                                        // iFrame entfernen
                                        uploadFrame.remove();
                                        // Formular Elemente anzeigen
                                        $('#$s_hideid').show();
                                        // Ajax Loader verstecken
                                        $('#$s_loaderid').hide();
                                    }, 1000);
                                });
                                // iFrame zum Body hinzufuegen
                                $('body:first').append(uploadFrame);
                                // Formular target zum iFrame setzen
                                submittingForm.attr('target', frameName);
                            });

                            $('#$s_buttonid').click(function(event) {
                                event.preventDefault();
                                // Formular abschicken
                                $('#$s_formid').submit();
                            });
                        });";
        cPage::s_addJavaScript($s_js);
        $this->end(false);
    }

    function uploadpic($i_sid, $s_name, $js_complete, $b_gallery = false, $s_class = '') {
        ob_start();
        $this->s_name = $s_name;
        $this->s_class = $s_class;
        $s_formid = $this->make_id('form');
        $s_hideid = $this->make_id('hide');
        $s_nameid = $this->make_id('name');
        $s_descripid = $this->make_id('textarea');
        $s_fileid = $this->make_id('file');
        $s_buttonid = $this->make_id('button');
        $s_loaderid = $this->make_id('loader');
        $b_gallery ? $s_g = '?g=1' : $s_g = '';
?>
        <form class="<?php echo $s_class ?>" action="upload_pic.php<?php echo $s_g; ?>" method="post" enctype="multipart/form-data" id="<?php echo
        $s_formid; ?>">
      <fieldset>
          <legend>Bild hochladen</legend>
          <label class="<?php echo $s_class ?>" for="<?php echo $s_nameid; ?>">Dateiname:</label>
          <input class="<?php echo $s_class ?>" type="text" name="name" id="<?php echo
        $s_nameid; ?>"/><br />
               <?php
               if ($b_gallery)
               {
                   echo '<label for="' . $s_descripid . '" class="' . $s_class .
                   '">Bildbeschreibung:</label>';
                   echo '<div class="clearer abstand"></div>';
                   $this->ckeditor('beschreibung');
                   echo '<p class="abstand"></p>';
               }
               else
               {
                   echo '<label for="' . $s_descripid . '" class="' . $s_class .
                   '">Bildbeschreibung:</label>
                        <textarea name="beschreibung" id="' . $s_descripid .
                   '" class="' . $s_class . '"></textarea><br>';
               }
               ?>
        <input class="<?php echo $s_class ?>" type="hidden" name="sid" value="<?php echo
               $i_sid; ?>" />
        <span id="<?php echo $s_hideid; ?>">
            <label class="<?php echo $s_class ?>" for="<?php echo $s_fileid; ?>">Datei:</label>
            <input class="<?php echo $s_class ?>" type="file" name="userfile" id="<?php echo
               $s_fileid; ?>"/><br />
            <input class="<?php echo $s_class ?> form-button" type="submit" name="<?php echo
                   $s_buttonid; ?>" value="Hochladen" id="<?php echo
                   $s_buttonid; ?>"/>
        </span>
    </fieldset>
</form>
<div id="<?php echo $s_loaderid; ?>" style="display:none;">L&auml;dt...<br/><img src="ajax-loader.gif" border="0"/></div>
<?php
                   $s_js = <<<HEREDOC
                $(function() {
                    // Formular abschicken
                    $('#$s_formid').submit(function(data) {
                        // Ajax Loader anzeigen
                        $('#$s_loaderid').show();
                        // Formular Elemente verstecken
                        $('#$s_hideid').hide();
                        // abschickendes Formular angeben
                        var submittingForm = $('#$s_formid');
                        // eindeutigen iFrame Namen generieren
                        var frameName = ("upload" + (new Date()).getTime());
                        // iFrame setzen
                        var uploadFrame = $('<iframe name="' + frameName + '"></iframe>');
                        // iFrame verstecken
                        uploadFrame.css("display", "none");
                        // iFrame Inhalt laden
                        uploadFrame.load(function(data) {
                            // Timeout festlegen
                            setTimeout(function() {
                                // Rueckgabewert abfragen
                                var response = uploadFrame.contents().find('#result').html();
                                $js_complete
                                // iFrame entfernen
                                uploadFrame.remove();
                                // Formular Elemente anzeigen
                                $('#$s_hideid').show();
                                // Ajax Loader verstecken
                                $('#$s_loaderid').hide();
                            }, 1000);
                        });
                        // iFrame zum Body hinzufuegen
                        $('body:first').append(uploadFrame);
                        // Formular target zum iFrame setzen
                        submittingForm.attr('target', frameName);
                    });

                    $('#$s_buttonid').click(function(event) {
                        event.preventDefault();
                        // Formular abschicken
                        $('#$s_formid').submit();
                    });
                });
HEREDOC;
                   cPage::s_addJavaScript($s_js);
                   $this->end(false);
               }

               function ckeditor($s_name, $s_value = '', $js_toolbar = '') {
                   //id holen
                   $s_id = $this->make_id('check');

                   // Daten zum berprfen setzen
                   $this->set_field($s_name, true, 'ckcontent', $s_name);
                   echo '<br>';
                   echo '<textarea class="' . $this->s_class . '" id="' . $s_id . '" name="' . $s_name .
                   '">' . $s_value . '</textarea><br>';
                   $s_js = '$(\'input.ckeditor, legend.ckeditor\').hide();
                            $(\'fieldset.ckeditor\').css({\'margin\':\'0px\', \'padding\': \'0px\',\'border\': \'none\'});
                            CKEDITOR.replace( \'' . $s_id . '\'';
                   if($js_toolbar != '') {
                        $s_js .= ',{toolbar: '.$js_toolbar.'}';
                   }
                   $s_js .= ');';
                   cPage::s_addJavaScript($s_js);
               }

               static function get_ckeditor($s_name) {
                   // berprfen ob Daten per Post gesendet wurden
                   if ('POST' != $_SERVER['REQUEST_METHOD'])
                   {
                       return false;
                   }
                   // Prfen ob der submitbutton gesetzt ist
                   if (isset($_POST[$s_name]) === false)
                   {
                       return false;
                   }
                   filter_var($_POST[$s_name], FILTER_SANITIZE_STRING);
                   $s_return = trim($_POST[$s_name]);
                   if ($s_return == '')
                   {
                       return 'Bitte geben Sie einen Inhalt an';
                   }
                   return $s_return;
               }

               function rightname($s_id) {
                   return str_replace(array('', '', '', '', ' '), array('ae', 'oe', 'ue', 'ss',
                       ''), mb_strtolower($s_id));
               }

               public function set_field($s_id, $b_pflicht, $s_type, $s_name) {
                   $this->a_fields[$s_id]['pflicht'] = $b_pflicht;
                   $this->a_fields[$s_id]['type'] = $s_type;
                   $this->a_fields[$s_id]['name'] = $s_name;
               }

               private function make_id($s_text) {
                   return $s_text . $this->s_name . ++$this->i_tagid;
               }

               private function get_onfocus($s_id, $s_beschreibung) {
                   if (trim($s_beschreibung) == '')
                   {
                       return 'onfocus="$(\'.description\').hide();"';
                   }
                   else
                   {
                       return 'onfocus="beschreibung(\'' . $s_id . '\',\'' . htmlentities($s_beschreibung) .
                       '\')"';
                   }
               }

           }
?>